from urllib.parse import urlparse

from kafka import KafkaProducer


def check(url):
    bootstrap_urls = url.split(",")
    bootstrap_parsed_urls = (urlparse(u) for u in bootstrap_urls)
    bootstrap_nodes = list(
        u.hostname + ":" + str(u.port or "9092") for u in bootstrap_parsed_urls
    )

    try:
        KafkaProducer(bootstrap_servers=bootstrap_nodes)
        return True
    except Exception as e:
        return False



import psycopg2


def check(url):
    if url.startswith("psql"):
        url = "postgres" + url[len("psql") :]

    try:
        conn = psycopg2.connect(url)
        cur = conn.cursor()
        cur.execute("SELECT NOW();")
        return True
    except:
        return False



import socket
from urllib.parse import urlparse


def check(url):
    filename = url[len("unix://") :]
    try:
        s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        s.connect(filename)
        return True
    except:
        return False



import importlib
import sys
import time
from urllib.parse import urlparse


class WaitForDep:
    def __init__(self, check_interval=1):
        self.check_interval = check_interval

    def wait(self, url):
        parsed_url = urlparse(url)
        scheme = parsed_url.scheme

        url_no_password = (
            url.replace(parsed_url.password, "*****") if parsed_url.password else url
        )

        try:
            module_name = f"wait_for_dep.checks.{scheme}"
            check_module = importlib.import_module(module_name)
            check = check_module.check
        except ImportError:
            print(
                "Unsupported scheme: {} in url: {}".format(scheme, url_no_password)
            )
            print("Using TCP for url: {}".format(url_no_password))
            module_name = f"wait_for_dep.checks.tcp"
            check_module = importlib.import_module(module_name)
            check = check_module.check

        i = 0
        while True:
            connected = check(url)
            if connected:
                break

            if i % 20 == 0:
                if i > 0:
                    sec = i * self.check_interval
                    print(
                        "Still waiting for {} ({}s elapsed)".format(
                            url_no_password, sec
                        )
                    )
                else:
                    print("Waiting for {}".format(url_no_password))
            i += 1

            time.sleep(self.check_interval)

        print("Successfully connected to {}".format(url_no_password))


def main():
    w = WaitForDep()
    args = sys.argv
    args.pop(0)

    for url in args:
        w.wait(url)


if __name__ == "__main__":
    main()



from urllib.parse import urlparse

import mysql.connector


def check(url):
    parsed_url = urlparse(url)
    config = {}

    if parsed_url.hostname:
        config["host"] = parsed_url.hostname

    if parsed_url.username:
        config["user"] = parsed_url.username

    if parsed_url.password:
        config["password"] = parsed_url.password

    if parsed_url.port:
        config["port"] = parsed_url.port

    if parsed_url.path:
        db = parsed_url.path
        if db[0] == "/":
            db = db[1:]
        if db:
            config["db"] = db

    try:
        conn = mysql.connector.connect(**config)

        cursor = conn.cursor()
        cursor.execute("SELECT VERSION();")
        return True
    except:
        return False



from urllib.parse import urlparse

import memcache


def check(url):
    parsed_url = urlparse(url)
    host = parsed_url.hostname
    port = parsed_url.port or "11211"

    try:
        mc = memcache.Client(["{}:{}".format(host, port)], debug=0)
        stats = mc.get_stats()
        return len(stats) > 0
    except:
        return False



from urllib.parse import parse_qs, urlparse

import pika


def check(url):
    parsed_url = urlparse(url)
    config = {}

    if parsed_url.hostname:
        config["host"] = parsed_url.hostname

    if parsed_url.username or parsed_url.password:
        creds = {}
        if parsed_url.username:
            creds["username"] = parsed_url.username

        if parsed_url.password:
            creds["password"] = parsed_url.password

        config["credentials"] = pika.PlainCredentials(**creds)

    if parsed_url.port:
        config["port"] = parsed_url.port

    if parsed_url.path:
        vhost = parsed_url.path
        if vhost[0] == "/":
            vhost = vhost[1:]
        if vhost:
            config["virtual_host"] = vhost

    required_queues = []
    required_exchanges = []

    if parsed_url.query:
        query = parse_qs(parsed_url.query)
        required_queues = query.get("require_queue", [])
        required_exchanges = query.get("require_exchange", [])

    try:
        connection = pika.BlockingConnection(pika.ConnectionParameters(**config))
        channel = connection.channel()

        for required_exchange in required_exchanges:
            channel.exchange_declare(exchange=required_exchange, passive=True)

        for required_queue in required_queues:
            channel.queue_declare(queue=required_queue, passive=True)

        return True
    except:
        return False



from pathlib import Path

from setuptools import find_packages, setup

BUNDLES = (
    "kafka",
    "memcached",
    "mysql",
    "postgres",
    "rabbitmq",
    "redis",
    "websockets",
    "mongodb",
)

# -*- Installation Requires -*-


def strip_comments(l):
    return l.split("#", 1)[0].strip()


def _pip_requirement(req, *root):
    if req.startswith("-r "):
        _, path = req.split()
        return reqs(*root, *path.split("/"))
    return [req]


def _reqs(*f):
    path = (Path.cwd() / "requirements").joinpath(*f)
    with path.open() as fh:
        reqs = [strip_comments(l) for l in fh.readlines()]
        return [_pip_requirement(r, *f[:-1]) for r in reqs if r]


def reqs(*f):
    return [req for subreq in _reqs(*f) for req in subreq]


def extras(*p):
    """Parse requirement in the requirements/extras/ directory."""
    return reqs("extras", *p)


def extras_require():
    """Get map of all extra requirements."""
    return {x: extras(x + ".txt") for x in BUNDLES}


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setup(
    name="wait-for-dep",
    entry_points={
        "console_scripts": [
            "wait-for-dep = wait_for_dep.wait_for_dep:main",
        ],
    },
    version="0.3.0",
    description="Waits for dependencies before running the app",
    url="http://github.com/wlatanowicz/wait-for-dep",
    author="Wiktor Latanowicz",
    author_email="wait-for-dep@wiktor.latanowicz.com",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(
        exclude=[
            "test*",
        ]
    ),
    zip_safe=False,
    install_requires=reqs("base.txt"),
    tests_require=reqs("tests.txt"),
    extras_require=extras_require(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



import requests


def check(url):
    try:
        response = requests.get(url, allow_redirects=True)
        if 200 <= response.status_code < 300:
            return True
        return False
    except:
        return False



from urllib.parse import urlparse

import redis


def check(url):
    parsed_url = urlparse(url)

    db_num = 0
    if parsed_url.path:
        url_path = parsed_url.path
        if url_path.startswith("/"):
            url_path = url_path[1:]
        db_num = int(url_path)

    try:
        redis.Redis(host=parsed_url.hostname, port=parsed_url.port or 6379, db=db_num)
        return True
    except Exception as e:
        return False
